package net.wanji.datacenter.cache;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossDirDataHistMapper;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.datacenter.common.Constants;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/20 10:41
 */
@Component
@Slf4j
public class CrossDirWeekAvgFlowCache implements CommandLineRunner {

    public static final Map<String, Double> crossDirWeekAvgFlowMap = new HashMap<>();

    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;

    private void getAvgFlow() {
        LocalDate currentDate = LocalDate.now();
        LocalDate lastDay = currentDate.minus(1, ChronoUnit.DAYS);
        LocalDate lastWeek = currentDate.minus(1, ChronoUnit.WEEKS);
        List<String> crossIds = CrossInfoCache.getSignalIds();
        for (String crossId : crossIds) {
            // 前一周
            LambdaQueryWrapper<CrossDirDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(CrossDirDataHistPO::getCrossId, crossId);
            queryWrapper.between(CrossDirDataHistPO::getStartTime, lastWeek, lastDay);
            List<CrossDirDataHistPO> crossDirDataHistPOS = crossDirDataHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(crossDirDataHistPOS)) {
                Map<Integer, List<CrossDirDataHistPO>> dirDataMap = crossDirDataHistPOS.stream().collect(Collectors.groupingBy(CrossDirDataHistPO::getDirType));
                if (!dirDataMap.isEmpty()) {
                    for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : dirDataMap.entrySet()) {
                        Integer dir = entry.getKey();
                        List<CrossDirDataHistPO> dirDataHistPOS = entry.getValue();
                        String currentCrossId = dirDataHistPOS.get(0).getCrossId();
                        OptionalDouble average = dirDataHistPOS.stream().map(CrossDirDataHistPO::getFlow).mapToInt(Integer::intValue).average();
                        double dirFlowAvg = 1.0;
                        if (average.isPresent()) {
                            dirFlowAvg = average.getAsDouble() == 0.0 ? 1.0 : average.getAsDouble();
                        }
                        crossDirWeekAvgFlowMap.put(currentCrossId.concat(Constants.underline).concat(String.valueOf(dir)), dirFlowAvg);
                    }
                }
            }
        }
    }

    @Override
    public void run(String... args) throws Exception {
        try {
            getAvgFlow();
        } catch (Exception e) {
            log.error("7天内路口方向平均流量获取失败", e);
            throw new Exception(e);
        }
        log.info("7天内路口方向平均流量：{}", crossDirWeekAvgFlowMap);
    }

}