package net.wanji.datacenter.cache;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossTurnDataHistMapper;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.datacenter.common.Constants;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.OptionalInt;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/19 15:33
 */
@Component
@Slf4j
public class CrossTurnWeekMaxFlowCache implements CommandLineRunner {

    public static final Map<String, Integer> crossTurnWeekMaxFlowMap = new HashMap<>();

    @Resource
    private CrossTurnDataHistMapper crossTurnDataHistMapper;

    private void getTurnMaxFlow() throws Exception {
        LocalDate currentDate = LocalDate.now();
        LocalDate lastDay = currentDate.minus(1, ChronoUnit.DAYS);
        LocalDate lastWeek = currentDate.minus(1, ChronoUnit.WEEKS);
        List<String> crossIds = CrossInfoCache.getSignalIds();
        for (String crossId : crossIds) {
            // 前一周
            LambdaQueryWrapper<CrossTurnDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(CrossTurnDataHistPO::getCrossId, crossId);
            queryWrapper.between(CrossTurnDataHistPO::getStartTime, lastWeek, lastDay);
            List<CrossTurnDataHistPO> crossTurnDataHistPOS = crossTurnDataHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(crossTurnDataHistPOS)) {
                Map<Integer, List<CrossTurnDataHistPO>> dirDataPOsMap = crossTurnDataHistPOS.stream().collect(Collectors.groupingBy(CrossTurnDataHistPO::getInDir));
                for (Map.Entry<Integer, List<CrossTurnDataHistPO>> entry : dirDataPOsMap.entrySet()) {
                    Integer dir = entry.getKey();
                    List<CrossTurnDataHistPO> dirHistPOS = entry.getValue();
                    if (!CollectionUtils.isEmpty(dirHistPOS)) {
                        Map<String, List<CrossTurnDataHistPO>> turnDataPOMap = dirHistPOS.stream().collect(Collectors.groupingBy(CrossTurnDataHistPO::getTurnType));
                        for (Map.Entry<String, List<CrossTurnDataHistPO>> turnEntry : turnDataPOMap.entrySet()) {
                            String turn = turnEntry.getKey();
                            List<CrossTurnDataHistPO> turnDataHistPOList = turnEntry.getValue();
                            if (!CollectionUtils.isEmpty(turnDataHistPOList)) {
                                OptionalInt max = turnDataHistPOList.stream().map(CrossTurnDataHistPO::getFlow).mapToInt(Integer::intValue).max();
                                if (max.isPresent()) {
                                    String key = crossId.concat(Constants.underline).concat(String.valueOf(dir)).concat(Constants.underline).concat(turn);
                                    crossTurnWeekMaxFlowMap.put(key, max.getAsInt());
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    public void run(String... args) throws Exception {
        try {
            getTurnMaxFlow();
        } catch (Exception e) {
            log.error("7天内路口方向转向最大流量获取失败", e);
            throw new Exception(e);
        }
        log.info("7天内路口方向转向最大流量：{}", crossTurnWeekMaxFlowMap);
    }

}