package net.wanji.datacenter.cache;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossDirDataHistMapper;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.datacenter.common.Constants;
import net.wanji.datacenter.kafka.ProducerHandler;
import net.wanji.datacenter.pojo.dto.CrossDirFreeFlowSpeedDTO;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/14 14:16
 */
@Component
@Slf4j
public class WeekDirFreeFlowSpeedCache implements CommandLineRunner {

    public static final Map<String, Double> crossDirFreeFlowSpeedMap = new HashMap<>();

    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;
    @Resource
    private ProducerHandler producerHandler;

    private void getDirWeekFreeFlowSpeed() throws Exception {
        LocalDate currentDate = LocalDate.now();
        LocalDate lastDay = currentDate.minus(1, ChronoUnit.DAYS);
        LocalDate lastWeek = currentDate.minus(1, ChronoUnit.WEEKS);
        List<String> crossIds = CrossInfoCache.getSignalIds();
        List<CrossDirFreeFlowSpeedDTO> crossDirFreeFlowSpeedDTOS = new ArrayList<>();
        for (String crossId : crossIds) {
            // 前一周
            LambdaQueryWrapper<CrossDirDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(CrossDirDataHistPO::getCrossId, crossId);
            queryWrapper.between(CrossDirDataHistPO::getStartTime, lastWeek, lastDay);
            List<CrossDirDataHistPO> crossDirDataHistPOS = crossDirDataHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(crossDirDataHistPOS)) {
                Map<Integer, List<CrossDirDataHistPO>> dirDataMap = crossDirDataHistPOS.stream().collect(Collectors.groupingBy(CrossDirDataHistPO::getDirType));
                if (!dirDataMap.isEmpty()) {
                    for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : dirDataMap.entrySet()) {
                        Integer dir = entry.getKey();
                        List<CrossDirDataHistPO> dirDataHistPOS = entry.getValue();
                        List<Double> weekSpeeds = dirDataHistPOS.stream().map(CrossDirDataHistPO::getSpeed).sorted().collect(Collectors.toList());
                        int startIndex = (int) (0.85 * weekSpeeds.size());
                        int endIndex = (int) (0.95 * weekSpeeds.size());
                        Double freeFlowSpeed = weekSpeeds.subList(startIndex, endIndex).stream().mapToDouble(Double::doubleValue).average().getAsDouble();
                        if (freeFlowSpeed == 0.0) {
                            freeFlowSpeed = 60.0;
                        }
                        crossDirFreeFlowSpeedMap.put(crossId.concat(Constants.underline).concat(String.valueOf(dir)), freeFlowSpeed);
                        CrossDirFreeFlowSpeedDTO crossDirFreeFlowSpeedDTO = new CrossDirFreeFlowSpeedDTO();
                        crossDirFreeFlowSpeedDTO.setCrossId(crossId);
                        crossDirFreeFlowSpeedDTO.setDir(dir);
                        crossDirFreeFlowSpeedDTO.setFreeFlowSpeed(freeFlowSpeed);
                        crossDirFreeFlowSpeedDTOS.add(crossDirFreeFlowSpeedDTO);
                    }
                }
            }
        }
        Map<String, List<CrossDirFreeFlowSpeedDTO>> map = new HashMap<>(1);
        map.put(Constants.CROSS_FREE_FLOW_SPEED_KEY, crossDirFreeFlowSpeedDTOS);
        producerHandler.send2FirstKafka(Constants.CROSS_FREE_FLOW_SPEED_TOPIC, map);
    }

    @Override
    public void run(String... args) throws Exception {
        try {
            getDirWeekFreeFlowSpeed();
        } catch (Exception e) {
            log.error("路口方向自由流速度失败", e);
            throw new Exception(e);
        }
        log.info("路口方向自由流速度：{}", crossDirFreeFlowSpeedMap);
    }

}
