package net.wanji.datacenter.cache;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossTurnDataHistMapper;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.datacenter.common.Constants;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.OptionalDouble;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/14 14:16
 */
@Component
@Slf4j
public class WeekTurnFreeFlowSpeedCache implements CommandLineRunner {

    public static final Map<String, Double> crossTurnFreeFlowSpeedMap = new HashMap<>();

    @Resource
    private CrossTurnDataHistMapper crossTurnDataHistMapper;

    private void getDirWeekFreeFlowSpeed() throws Exception {
        LocalDate currentDate = LocalDate.now();
        LocalDate lastDay = currentDate.minus(1, ChronoUnit.DAYS);
        LocalDate lastWeek = currentDate.minus(1, ChronoUnit.WEEKS);
        List<String> crossIds = CrossInfoCache.getSignalIds();
        for (String crossId : crossIds) {
            // 前一周
            LambdaQueryWrapper<CrossTurnDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(CrossTurnDataHistPO::getCrossId, crossId);
            queryWrapper.between(CrossTurnDataHistPO::getStartTime, lastWeek, lastDay);
            List<CrossTurnDataHistPO> crossTurnDataHistPOS = crossTurnDataHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(crossTurnDataHistPOS)) {
                Map<Integer, List<CrossTurnDataHistPO>> dirDataMap = crossTurnDataHistPOS.stream().collect(Collectors.groupingBy(CrossTurnDataHistPO::getInDir));
                for (Map.Entry<Integer, List<CrossTurnDataHistPO>> dirEntry : dirDataMap.entrySet()) {
                    Integer dir = dirEntry.getKey();
                    List<CrossTurnDataHistPO> dirTurnDataHistPOS = dirEntry.getValue();
                    Map<String, List<CrossTurnDataHistPO>> turnDataMap = dirTurnDataHistPOS.stream().collect(Collectors.groupingBy(CrossTurnDataHistPO::getTurnType));
                    if (!turnDataMap.isEmpty()) {
                        for (Map.Entry<String, List<CrossTurnDataHistPO>> entry : turnDataMap.entrySet()) {
                            String turn = entry.getKey();
                            List<CrossTurnDataHistPO> turnDataHistPOS = entry.getValue();
                            String currentCrossId = turnDataHistPOS.get(0).getCrossId();
                            List<Double> weekTurnSpeeds = turnDataHistPOS.stream().map(CrossTurnDataHistPO::getSpeed).sorted().collect(Collectors.toList());
                            int startIndex = (int) (0.85 * weekTurnSpeeds.size());
                            int endIndex = (int) (0.95 * weekTurnSpeeds.size());
                            if (startIndex == endIndex) {
                                startIndex = startIndex - 1;
                            }
                            OptionalDouble average = weekTurnSpeeds.subList(startIndex, endIndex).stream().mapToDouble(Double::doubleValue).average();
                            if (average.isPresent()) {
                                double turnFreeFlowSpeed = average.getAsDouble();
                                String key = currentCrossId.concat(Constants.underline).concat(String.valueOf(dir)).concat(Constants.underline).concat(turn);
                                crossTurnFreeFlowSpeedMap.put(key, turnFreeFlowSpeed);
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    public void run(String... args) throws Exception {
        try {
            getDirWeekFreeFlowSpeed();
        } catch (Exception e) {
            log.error("路口转向自由流速度失败", e);
            throw new Exception(e);
        }
        log.info("路口转向自由流速度：{}", crossTurnFreeFlowSpeedMap);
    }

}
