package net.wanji.datacenter.pojo.convert;

import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.databus.po.*;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import org.springframework.stereotype.Component;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.Random;

/**
 * @author duanruiming
 * @date 2023/03/11 14:04
 */
@SuppressWarnings("all")
@Component
public class LanePeriodicDataEventListConvert {

    public CrossDataRealtimePO convert2CrossDataRealTimePO(LanePeriodicDataDTO.EventList eventList, CrossBaseLaneInfoPO laneInfoPO) {
        String id = laneInfoPO.getId();
        String crossId = laneInfoPO.getCrossId();
        // todo 路口状态
        int trafficFlow = eventList.getTrafficFlow();
        double meanV = eventList.getMeanV() == 0.0 ? 50.0 : eventList.getMeanV();
        double staticQueueLengthMax = eventList.getStaticQueueLengthMax();
        double staticQueueLengthMin = eventList.getStaticQueueLengthMin() == 0.0 ? 10 : eventList.getStaticQueueLengthMin();
        double stopTimes = eventList.getMeanStopsNumber();
        double meanDelay = eventList.getMeanDelay();
        Integer meanDelayInt = Double.valueOf(meanDelay).intValue();
        double laneSaturation = eventList.getLaneSaturation();
        String dir = String.valueOf(laneInfoPO.getDir());
        double overflowRate = eventList.getOverflowRate();
        double greenLightEfficiency = eventList.getGreenLightEfficiency();

        CrossDataRealtimePO crossDataRealTimePO = new CrossDataRealtimePO();
        crossDataRealTimePO.setCrossId(crossId);
        crossDataRealTimePO.setStatus(eventList.getTrafficInformation()); // 路口状态
        crossDataRealTimePO.setType(1); // 1常规 2异常
        crossDataRealTimePO.setDuration(5); // todo 通过第二批次判断
        crossDataRealTimePO.setIsUnbalance(0);
        crossDataRealTimePO.setUnbalanceIndex(0.0);
        crossDataRealTimePO.setUnbalanceDirs(dir);
        crossDataRealTimePO.setIsCongestion(0);
        crossDataRealTimePO.setCongestionIndex(0.0); // 拥堵指数=
        crossDataRealTimePO.setCongestionDirs(dir);
        crossDataRealTimePO.setIsSpillover(0);
        crossDataRealTimePO.setSpilloverIndex(0.0);
        crossDataRealTimePO.setSpilloverDirs(dir);
        crossDataRealTimePO.setTrafficIndex(1.0);// 交通指数=所有失衡拥堵溢出指数平均
        crossDataRealTimePO.setFlow(trafficFlow);
        crossDataRealTimePO.setFlowRate(Double.valueOf(trafficFlow / 5 * 60)); // 每个车道流率
        crossDataRealTimePO.setSpeed(meanV);
        crossDataRealTimePO.setQueueLength(staticQueueLengthMax);
        crossDataRealTimePO.setStopTimes(stopTimes);
        crossDataRealTimePO.setDelayTime(meanDelayInt);
        crossDataRealTimePO.setSturation(laneSaturation);
        crossDataRealTimePO.setEffusionRate(overflowRate);
        crossDataRealTimePO.setGreenLightEfficiency(greenLightEfficiency);
        crossDataRealTimePO.setNoStopRate(eventList.getLaneNoStopRate());
        crossDataRealTimePO.setOneStopRate(eventList.getLaneOneStopRate());
        crossDataRealTimePO.setTwoStopRate(eventList.getLaneTwoStopRate());
        crossDataRealTimePO.setThreeStopRate(eventList.getLaneThreeStopRate());
        return crossDataRealTimePO;
    }

    public CrossDirDataRealtimePO convert2CrossDirDataRealTimePO(LanePeriodicDataDTO.EventList eventList, CrossBaseLaneInfoPO laneInfoPO) {
        String crossId = laneInfoPO.getCrossId();
        // todo 路口状态
        int trafficFlow = eventList.getTrafficFlow();
        double meanV = eventList.getMeanV();
        double staticQueueLengthMax = eventList.getStaticQueueLengthMax();
        double staticQueueLengthMin = eventList.getStaticQueueLengthMin();
        double stopTimes = eventList.getMeanStopsNumber();
        double meanDelay = eventList.getMeanDelay();
        Integer meanDelayInt = Double.valueOf(meanDelay).intValue();
        double laneSaturation = eventList.getLaneSaturation();
        Integer dir = laneInfoPO.getDir();
        Integer type = laneInfoPO.getType();
        double overflowRate = eventList.getOverflowRate();
        double greenLightEfficiency = eventList.getGreenLightEfficiency();

        CrossDirDataRealtimePO crossDirDataRealTimePO = new CrossDirDataRealtimePO();
        // 路口方向ID（路口ID_方向_进出口_主辅路序号）
        crossDirDataRealTimePO.setId(String.valueOf(dir).concat(String.valueOf(type)));
        crossDirDataRealTimePO.setDirType(dir);
        crossDirDataRealTimePO.setInOutType(type);
        crossDirDataRealTimePO.setCrossId(crossId);
        crossDirDataRealTimePO.setLength(staticQueueLengthMax);
        crossDirDataRealTimePO.setStatus(eventList.getTrafficInformation());
        crossDirDataRealTimePO.setTrafficIndex(0000.00);
        crossDirDataRealTimePO.setCapacity(trafficFlow);
        crossDirDataRealTimePO.setDuration(5);
        crossDirDataRealTimePO.setFlow(trafficFlow);
        crossDirDataRealTimePO.setSpeed(meanV);
        crossDirDataRealTimePO.setQueueLength(staticQueueLengthMax);
        crossDirDataRealTimePO.setStopTimes(stopTimes);
        crossDirDataRealTimePO.setDelayTime(meanDelayInt);
        crossDirDataRealTimePO.setSturation(laneSaturation);
        crossDirDataRealTimePO.setEffusionRate(eventList.getOverflowRate());
        crossDirDataRealTimePO.setNoStopRate(eventList.getLaneNoStopRate());
        crossDirDataRealTimePO.setOneStopRate(eventList.getLaneOneStopRate());
        crossDirDataRealTimePO.setTwoStopRate(eventList.getLaneTwoStopRate());
        crossDirDataRealTimePO.setThreeStopRate(eventList.getLaneThreeStopRate());
        crossDirDataRealTimePO.setEffusionRate(overflowRate);
        crossDirDataRealTimePO.setGreenLightEfficiency(greenLightEfficiency);
        return crossDirDataRealTimePO;
    }

    public CrossTurnDataRealtimePO convert2CrossTurnDataRealtimePO(LanePeriodicDataDTO.EventList eventList, CrossBaseLaneInfoPO laneInfoPO) {
        String crossId = laneInfoPO.getCrossId();
        // todo 路口状态
        int trafficFlow = eventList.getTrafficFlow();
        double meanV = eventList.getMeanV();
        double staticQueueLengthMax = eventList.getStaticQueueLengthMax();
        double stopTimes = eventList.getMeanStopsNumber();
        double meanDelay = eventList.getMeanDelay();
        Integer meanDelayInt = Double.valueOf(meanDelay).intValue();
        double laneSaturation = eventList.getLaneSaturation();
        Integer dir = laneInfoPO.getDir();
        Integer type = laneInfoPO.getType();
        Integer turn = laneInfoPO.getTurn();
        double overflowRate = eventList.getOverflowRate();
        double greenLightEfficiency = eventList.getGreenLightEfficiency();

        CrossTurnDataRealtimePO crossTurnDataRealtimePO = new CrossTurnDataRealtimePO();
        crossTurnDataRealtimePO.setId(crossId.concat("").concat(String.valueOf(turn)));
        crossTurnDataRealtimePO.setTurnType(TurnConvertEnum.getCodeByKey(turn));
        crossTurnDataRealtimePO.setInDir(dir);
        Integer outdir = dir + 4 > 8 ? dir + 4 - 8 : dir + 4;
        crossTurnDataRealtimePO.setOutDir(outdir);
        crossTurnDataRealtimePO.setCrossId(crossId);
        crossTurnDataRealtimePO.setFlow(trafficFlow);
        crossTurnDataRealtimePO.setStatus(eventList.getTrafficInformation());
        crossTurnDataRealtimePO.setTrafficIndex((double) new Random().nextInt(10));
        crossTurnDataRealtimePO.setSpeed(meanV);
        crossTurnDataRealtimePO.setInSpeed(meanV);
        crossTurnDataRealtimePO.setOutSpeed(meanV);
        crossTurnDataRealtimePO.setQueueLength(staticQueueLengthMax);
        crossTurnDataRealtimePO.setStopTimes(stopTimes);
        crossTurnDataRealtimePO.setDelayTime(meanDelayInt);
        crossTurnDataRealtimePO.setSturation(laneSaturation);
        crossTurnDataRealtimePO.setVehheadDist(eventList.getMeanLen());
        crossTurnDataRealtimePO.setVehheadTime(eventList.getTimeHeadwaySectionMean());
        crossTurnDataRealtimePO.setQuality(10.0);
        crossTurnDataRealtimePO.setNoStopRate(eventList.getLaneNoStopRate());
        crossTurnDataRealtimePO.setOneStopRate(eventList.getLaneOneStopRate());
        crossTurnDataRealtimePO.setTwoStopRate(eventList.getLaneTwoStopRate());
        crossTurnDataRealtimePO.setThreeStopRate(eventList.getLaneThreeStopRate());
        crossTurnDataRealtimePO.setEffusionRate(overflowRate);
        crossTurnDataRealtimePO.setGreenLightEfficiency(greenLightEfficiency);
        return crossTurnDataRealtimePO;
    }

    public CrossLaneDataRealTimePO convert2CrossLaneDataRealTimePO(LanePeriodicDataDTO.EventList eventList, CrossBaseLaneInfoPO laneInfoPO, Date startDate) {
        DecimalFormat decimalFormat = new DecimalFormat("#.##");
        String crossId = laneInfoPO.getCrossId();
        // todo 路口状态
        int trafficFlow = eventList.getTrafficFlow();
        double meanV = Double.valueOf(decimalFormat.format(eventList.getMeanV()));
        double staticQueueLengthMax = Double.valueOf(decimalFormat.format(eventList.getStaticQueueLengthMax()));
        double staticQueueLengthMin = Double.valueOf(decimalFormat.format(eventList.getStaticQueueLengthMin()));
        double stopTimes = eventList.getMeanStopsNumber();
        double meanDelay = eventList.getMeanDelay();
        Integer meanDelayInt = Double.valueOf(meanDelay).intValue();
        double laneSaturation = Double.valueOf(decimalFormat.format(eventList.getLaneSaturation()));
        Integer batchTime = Integer.valueOf(String.valueOf(startDate.getTime()).substring(0, 10));
        Double noStopRate = Double.valueOf(decimalFormat.format(eventList.getLaneNoStopRate()));
        Double oneStopRate = Double.valueOf(decimalFormat.format(eventList.getLaneOneStopRate()));
        Double twoStopRate = Double.valueOf(decimalFormat.format(eventList.getLaneTwoStopRate()));
        Double threeStopRate = Double.valueOf(decimalFormat.format(eventList.getLaneThreeStopRate()));
        Double timeHeadwaySectionMean = Double.valueOf(decimalFormat.format(eventList.getTimeHeadwaySectionMean()));
        Double meanLen = Double.valueOf(decimalFormat.format(eventList.getMeanLen()));
        Double overflowRate = Double.valueOf(decimalFormat.format(eventList.getOverflowRate()));
        Double greenLightEfficiency = Double.valueOf(decimalFormat.format(eventList.getGreenLightEfficiency()));
        Double timeOccupancy = Double.valueOf(decimalFormat.format(eventList.getTimeOccupancy()));
        Double vehicleNumsRatioMean = Double.valueOf(decimalFormat.format(eventList.getVehicleNumsRatioMean()));
        Double vehicleLengthRatioMean = Double.valueOf(decimalFormat.format(eventList.getVehicleLengthRatioMean()));

        CrossLaneDataRealTimePO crossLaneDataRealTimePO = new CrossLaneDataRealTimePO();
        crossLaneDataRealTimePO.setId(laneInfoPO.getId());
        crossLaneDataRealTimePO.setCrossId(crossId);
        crossLaneDataRealTimePO.setFlow(trafficFlow);
        crossLaneDataRealTimePO.setSpeed(meanV);
        crossLaneDataRealTimePO.setInSpeed(meanV);
        crossLaneDataRealTimePO.setOutSpeed(meanV);
        crossLaneDataRealTimePO.setQueueLength(staticQueueLengthMax);
        crossLaneDataRealTimePO.setStopTimes(stopTimes);
        crossLaneDataRealTimePO.setDelayTime(meanDelayInt);
        crossLaneDataRealTimePO.setCapacity(trafficFlow);
        crossLaneDataRealTimePO.setSturation(laneSaturation);
        crossLaneDataRealTimePO.setVehheadDist(meanLen);
        crossLaneDataRealTimePO.setVehheadTime(timeHeadwaySectionMean);
        crossLaneDataRealTimePO.setQuality(10.0);
        crossLaneDataRealTimePO.setNoStopRate(noStopRate);
        crossLaneDataRealTimePO.setOneStopRate(oneStopRate);
        crossLaneDataRealTimePO.setTwoStopRate(twoStopRate);
        crossLaneDataRealTimePO.setThreeStopRate(threeStopRate);
        crossLaneDataRealTimePO.setBatchTime(batchTime);
        crossLaneDataRealTimePO.setStartTime(startDate);
        crossLaneDataRealTimePO.setGmtCreate(new Date());
        crossLaneDataRealTimePO.setGmtModified(new Date());
        crossLaneDataRealTimePO.setEffusionRate(overflowRate);
        crossLaneDataRealTimePO.setGreenLightEfficiency(greenLightEfficiency);
        crossLaneDataRealTimePO.setTimeOccupancy(timeOccupancy);
        crossLaneDataRealTimePO.setVehicleNumsRatioMean(vehicleNumsRatioMean);
        crossLaneDataRealTimePO.setVehicleLengthRatioMean(vehicleLengthRatioMean);
        crossLaneDataRealTimePO.setNonMotorFlow(eventList.getNonMotorFlow());
        crossLaneDataRealTimePO.setV85(eventList.getV85());
        crossLaneDataRealTimePO.setTrafficFlowA(eventList.getTrafficFlowA());
        crossLaneDataRealTimePO.setTrafficFlowB(eventList.getTrafficFlowB());
        crossLaneDataRealTimePO.setTrafficFlowC(eventList.getTrafficFlowC());
        return crossLaneDataRealTimePO;
    }
}
